@@ -235,46 +235,51 @@ static void mm_insert_chunk(MM *mm, mem_chunk *mcInsert)
 static mem_chunk *mm_retrieve_chunk(MM *mm, size_t size)
 {
     mem_chunk *mc;
-    mem_chunk *mcMinLast;
-    mem_chunk *mcMin;
+    mem_chunk **pmcMin;
+    mem_chunk *mcRes;
     size_t sMin;
     size_t s;
 
     if (mm->mp_freechunks.mc_usize == 0)
         return NULL;
     mm_core_lock((void *)mm, MM_LOCK_RW);
+    /* find best-fitting chunk */
+    pmcMin = NULL;
+    sMin = mm->mp_size; /* maximum possible */
     mc = &(mm->mp_freechunks);
-    mcMinLast = mc;
-    mcMin     = NULL;
-    sMin      = mm->mp_size; /* maximum possible */
     while (mc->mc_u.mc_next != NULL) {
-        mc = mc->mc_u.mc_next;
-        if (mc->mc_size >= size && mc->mc_size < sMin) {
-            mcMin = mc;
-            sMin  = mc->mc_size;
+        s = mc->mc_u.mc_next->mc_size;
+        if (s >= size && s < sMin) {
+            pmcMin = &(mc->mc_u.mc_next);
+            sMin = s;
+            if (s == size)
+                break;
         }
-        mcMinLast = mc;
+        mc = mc->mc_u.mc_next;
     }
-    if (mcMin != NULL) {
-        if (mcMin->mc_size > size+1024) {
+    /* create result chunk */
+    if (pmcMin == NULL)
+        mcRes = NULL;
+    else {
+        mcRes = *pmcMin;
+        if (mcRes->mc_size >= (size + min_of(2*size,128))) {
             /* split out in part */
-            s = mcMin->mc_size - size;
-            mcMin->mc_size = size;
-
+            s = mcRes->mc_size - size;
+            mcRes->mc_size = size;
             /* add back remaining chunk part as new chunk */
-            mc = (mem_chunk *)((char *)mcMin + size);
+            mc = (mem_chunk *)((char *)mcRes + size);
             mc->mc_size = s;
-            mc->mc_u.mc_next = mcMin->mc_u.mc_next;
-            mcMinLast->mc_u.mc_next = mc;
+            mc->mc_u.mc_next = mcRes->mc_u.mc_next;
+            *pmcMin = mc;
         }
         else {
             /* split out as a whole */
-            mcMinLast->mc_u.mc_next = mcMin->mc_u.mc_next;
+            *pmcMin = mcRes->mc_u.mc_next;
             mm->mp_freechunks.mc_usize--;
         }
     }
     mm_core_unlock((void *)mm);
-    return mcMin;
+    return mcRes;
 }
 
 /*
@@ -291,9 +296,11 @@ void *mm_malloc(MM *mm, size_t usize)
     size = mm_core_align2word(SIZEOF_mem_chunk+usize);
     if ((mc = mm_retrieve_chunk(mm, size)) != NULL) {
         mc->mc_usize = usize;
-        return mc;
+        return &(mc->mc_u.mc_base.mw_cp);
     }
+    mm_core_lock((void *)mm, MM_LOCK_RW);
     if ((mm->mp_size - mm->mp_offset) < size) {
+        mm_core_unlock((void *)mm);
         ERR(MM_ERR_ALLOC, "Out of memory");
         errno = ENOMEM;
         return NULL; 
@@ -303,6 +310,7 @@ void *mm_malloc(MM *mm, size_t usize)
     mc->mc_usize = usize;
     vp = (void *)&(mc->mc_u.mc_base.mw_cp);
     mm->mp_offset += size;
+    mm_core_unlock((void *)mm);
     return vp;
 }
 
