@@ -172,7 +172,7 @@ size_t mm_core_align2word(size_t size)
 
 size_t mm_core_maxsegsize(void)
 {
-    return MM_SHM_MAXSEGSIZE;
+    return mm_core_align2page((MM_SHM_MAXSEGSIZE-SIZEOF_mem_core)-mm_core_pagesize());
 }
 
 /*
@@ -209,13 +209,16 @@ void *mm_core_create(size_t usersize, const char *file)
 #endif
     char filename[MM_MAXPATH];
 
-    mm_core_init();
-    if (usersize <= 0 || usersize >= mm_core_maxsegsize())
-        usersize = mm_core_maxsegsize();
+    if (usersize <= 0 || usersize > mm_core_maxsegsize()) {
+        errno = EINVAL;
+        return NULL;
+    }
     if (file == NULL) {
         sprintf(filename, MM_CORE_DEFAULT_FILE, (int)getpid());
         file = filename;
     }
+
+    mm_core_init();
     size = mm_core_align2page(usersize+SIZEOF_mem_core);
 
 #if defined(MM_SHMT_MMPOSX) || defined(MM_SHMT_MMFILE)
