@@ -58,6 +58,13 @@ int MM_create(size_t size, const char *file)
     return TRUE;
 }
 
+int MM_permission(mode_t mode, uid_t owner, gid_t group)
+{
+    if (mm_global != NULL)
+        return -1;
+    return mm_permission(mm_global, mode, owner, group);
+}
+
 void MM_destroy(void)
 {
     if (mm_global == NULL)
